<?php

App::uses('AppComponent', 'Controller/Component');

class ProductsComponent extends AppComponent {

	/**
	 * get a product listing
	 *
	 * @param 	array 	query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @param   array   An array of order data (direction and currencyId)
	 * @return 	array 	Array of results
	 */
	public function listing($params = array(), $paginate = true, $orderData = null) {
		$Product = EvClassRegistry::init('EvShop.Product');
		$params = Hash::merge(
			array(
				'joins' => array(
					array(
						'table' => 'ev_shop_categories_products',
						'alias' => 'CategoriesProduct',
						'conditions' => array(
							'Product.id = CategoriesProduct.product_id'
						)
					)
				),
				'conditions' => array(
					'Product.is_active' => 1
				),
				'order' => 'CategoriesProduct.sequence ASC, Product.name ASC',
				'contain' => array(
					'ListingImage',
					'Variant' => array(
						'order' => 'Variant.sequence ASC',
						'VariantPricing',
						'Inventory'
					)
				),
				'group' => 'Product.id'
			),
			$params
		);

		$params = $this->_addAdditionalParamsToListing($Product, $params, $orderData);

		//Link Variant Inventory
		$Product->Variant->linkInventory();

		// debug($params);
		// die;

		if ($paginate && ! empty($this->_controller)) {
			$this->_controller->Paginator->settings = $params;
			$result = $this->_controller->Paginator->paginate(
				$Product
			);
		} else {
			$result = $Product->find('all', $params);
		}

		// debug($Product->getDataSource()->getLog(false, false));
		// die;

		$this->_addTaxToProducts($result);

		return $result;
	}

	/**
	 * get a featured product listing
	 *
	 * @param 	array 	query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @return 	array 	Array of results
	 */
	public function featured($params = array(), $paginate = false) {
		$params = Hash::merge(
			array(
				'conditions' => array(
					'Product.is_featured' => 1
				)
			),
			$params
		);

		return $this->listing($params, $paginate);
	}

	/**
	 * get a listing by category
	 *
	 * @param 	int|array 	Category ID or array of category IDs
	 * @param 	array 		query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @param   array       An array to pass through to listing that if set will contain the direction of ordering and currency_id
	 * @return 	array 		Array of results
	 */
	public function listingByCategory($categoryId, $params = array(), $paginate = true, $orderData = null) {
		$defaults = array(
			'conditions' => array(
				'CategoriesProduct.category_id' => $categoryId
			)
		);
		// if it's many categories, and the group by and9
		// also contain the categories so we know which is which
		if (is_array($categoryId)) {
			$defaults['contain']['CategoriesProduct'] = array(
				'limit' => 1,
				'Category'
			);
		}

		$params = Hash::merge(
			$defaults,
			$params
		);

		return $this->listing($params, $paginate, $orderData);
	}

	/**
	 * get a listing by a brand
	 *
	 * @param 	int|array 	brand ID or array of brand IDs
	 * @param 	array 		query / paginate params
	 * @param 	bool 	Paginate results or not?
	 * @return 	array 		Array of results
	 */
	public function listingByBrand($brandId, $params = array(), $paginate = true) {
		$defaults = array(
			'conditions' => array(
				'Product.brand_id' => $brandId
			)
		);

		// if it's many categories, and the group by and
		// also contain the categories so we know which is which
		if (is_array($brandId)) {
			$defaults['contain'][] = 'Brand';
		}

		$params = Hash::merge(
			$defaults,
			$params
		);

		return $this->listing($params, $paginate);
	}

	/**
	 * Check if specific ordering data is provided and if so call the function that adds the correct parameters to the
	 * query. Extend to add add your own parameters to the listing query.
	 * @param array $currentParams The current parameters of the query to change. Normally the defaults set in listing()
	 * @param array $orderData     The ordering data (direction and currencyId if it is set)
	 * @param Model $queryModel    The model that is being queried on, listing uses Product
	 */
	protected function _addAdditionalParamsToListing(&$queryModel, $currentParams, $orderData) {
		//Go through the passed ordering parameters and add it to the pagination query
		if (isset($orderData['price']) && !empty($orderData['price'])) {
			$currentParams = $this->_addPriceParamsToListing($currentParams, $orderData['price'], $queryModel);
		}

		if (isset($orderData['brand']) && !empty($orderData['brand'])) {
			$currentParams = $this->_addBrandParamsToListing($currentParams, $orderData['brand'], $queryModel);
		}

		if (isset($orderData['option']) && !empty($orderData['option'])) {
			$currentParams = $this->_addOptionParamsToListing($currentParams, $orderData['option'], $queryModel);
		}

		return $currentParams;
	}

	/**
	 * Adds joins and order to the pagination query parameters. Defaults to ordering
	 * in ascending order. The ordering will chose the lowest price between the price and sale_price
	 * to order by.
	 * @param array $currentParams The current parameters of the query to change. Normally the defaults set in listing()
	 * @param array $orderPrice     The ordering data (direction and currencyId if it is set)
	 * @param Model $queryModel    The model that is being queried on, listing uses Product
	 */
	protected function _addPriceParamsToListing($currentParams, $orderPrice, &$queryModel) {
		//Alias named to "VariantForPrice" to avoid join clashes when multiple joins for Variant are added.
		$currentParams['joins'][] = [
			'table' => 'ev_shop_variants',
			'alias' => 'VariantForPrice',
			'conditions' => [
				'VariantForPrice.product_id = Product.id'
			]
		];

		$variantPricingJoin = [
			'table' => 'ev_shop_variant_pricings',
			'alias' => 'VariantPricing',
			'conditions' => [
				'VariantPricing.variant_id = VariantForPrice.id'
			],
			'type' => 'left'
		];

		if (isset($orderPrice['currencyId'])) {
			$variantPricingJoin['conditions'] = 'VariantPricing.currency_id = ' . $orderPrice['currencyId'];
		}

		$currentParams['joins'][] = $variantPricingJoin;

		$orderDirection = 'ASC';
		if (isset($orderPrice['direction'])) {
			$orderDirection = $orderPrice['direction'];
		}

		$currentParams['order'] = "MIN(IF(VariantPricing.sale_price IS NULL, VariantPricing.price, VariantPricing.sale_price)) $orderDirection";

		return $currentParams;
	}

	/**
	 * Adds joins and conditions to the listing params so that products are selected only if they belong to a single
	 * brand. Brands are provided throught the order data as a list of the ids that the products need to match to.
	 * @param array $currentParams The current parameters of the query to change. Normally the defaults set in listing()
	 * @param array $orderBrands     The ordering data (list of brand ids)
	 * @param Model $queryModel    The model that is being queried on, listing uses Product
	 */
	protected function _addBrandParamsToListing($currentParams, $orderBrands, &$queryModel) {
		$currentParams['joins'][] = [
			'table' => 'ev_shop_brands',
			'alias' => 'Brand',
			'conditions' => [
				'Brand.id = Product.brand_id'
			]
		];

		$currentParams['conditions']['Brand.name'] = $orderBrands;

		return $currentParams;
	}

	/**
	 * Adds joins and conditions to the listing params so that products are selected only if they belong to a single
	 * option. Options are provided throught the order data as a list of the ids that the products need to match to.
	 * @param array $currentParams The current parameters of the query to change. Normally the defaults set in listing()
	 * @param array $orderOptions     The ordering data (Array of option groups as keys to a list of options in that group)
	 * @param Model $queryModel    The model that is being queried on, listing uses Product
	 */
	protected function _addOptionParamsToListing($currentParams, $orderOptions, &$queryModel) {
		if (!empty($orderOptions)) {
			//Alias named to "VariantForOption" to avoid join clashes when multiple joins for Variant are added.
			$currentParams['joins'][] = [
				'table' => 'ev_shop_variants',
				'alias' => 'VariantForOption',
				'conditions' => [
					'VariantForOption.product_id = Product.id'
				]
			];

			//To test upon each option group for the options that have been selected, a join needs to be made
			//for each option group. The link table also needs to be joined on for each option group so that
			//the same variant isn't being used for the same conditions. The loop below just goes through each
			//option group supplied and appends it's name to each join so that they don't clash with seperate joins
			//or other selected option joins.
			foreach ($orderOptions as $optionGroup => $optionArray) {
				$currentParams['joins'][] = [
					'table' => 'ev_shop_options_variants',
					'alias' => 'OptionVariant' . $optionGroup,
					'conditions' => [
						'OptionVariant' . $optionGroup . '.variant_id = VariantForOption.id'
					]
				];

				$currentParams['joins'][] = [
					'table' => 'ev_shop_options',
					'alias' => 'Option' . $optionGroup,
					'conditions' => [
						'Option' . $optionGroup . '.id = OptionVariant' . $optionGroup . '.option_id',
						'Option' . $optionGroup . '.name' => $optionArray,
					]
				];
			}
		}

		return $currentParams;
	}

	/**
	 * Adds the _incTax fields to the query results. Checks that the EvTax plugin is loaded if not then
	 * it is assumed that the tax will be added manually from whatever peeps be using.
	 * @param array $products The results of the product query containing each product to add tax to.
	 */
	protected function _addTaxToProducts($products) {
		if (CakePlugin::loaded('EvTax')) {
			$TaxLevelModel = EvClassRegistry::init('EvTax.TaxLevel');

			$VariantModel = EvClassRegistry::init('EvShop.Variant');
			$VariantPricingModel = EvClassRegistry::init('EvShop.VariantPricing');
			$VariantModel->linkInventory();

			if (! empty($products)) {
				foreach ($products as $key => $item) {
					$taxLevel = array();
					if (isset($item['Product']['tax_level_id']) && $item['Product']['tax_level_id'] > 0) {

						$taxLevel = $TaxLevelModel->find('first', array(
							'conditions' => array(
								'id' => $item['Product']['tax_level_id']
							)
						));

						if (isset($item['Variant']) && ! empty($item['Variant'])) {
							foreach ($item['Variant'] as $vKey => $variant) {

								if (! empty($taxLevel['TaxLevel'])) {

									if (isset($variant['VariantPricing']) && ! empty($variant['VariantPricing'])) {

										foreach ($variant['VariantPricing'] as $pricingKey => $pricing) {
											$variant['VariantPricing'][$pricingKey]['TaxLevel'] = $taxLevel['TaxLevel'];
										}
									}
								}

								if (isset($result[$key]['Variant'][$vKey])) {
									$products[$key]['Variant'][$vKey] = $VariantPricingModel->calculateTax($variant, 'Variant');
								}
							}
						}
					}
				}
			}
		}

		return $products;
	}
}
